"""
"""

from Namespace import *

class ICredentials(Interface):

    def getUser():
        """
        return the user name for this cred.
        """

    def getPassword():
        """
        return the password for this cred.
        """

class IHost(Interface):

    def getHost():
        """
        return a FQDN or ip
        """

class IProtocol(Interface):
    """ represents a transport protocol """

    def execute(self, target, structure):
        """
        deploy structure to target
        """

class IDeploymentTarget(ICredentials, IHost):


    def getDirectory():
        """
        target directory
        """

    def getProtocol():
        """
        return the protocol to be used with
        this object
        """

class CredentialError(Exception): pass
class ProtocolError(Exception): pass


class DeploymentTarget(SimpleItem):

    __implements__ = IDeploymentTarget

    meta_type = 'Deployment Target'
    
    security = ClassSecurityInfo()
    
    manage_options = (
        
        {'label':'Settings',
         'action':'deployment_settings'},

        {'label':'Targets',
         'action':'../manage_main'},

        {'label':'Tool',
         'action':'../../overview'}
        )

    deployment_settings = DTMLFile('ui/DeploymentTargetSettingsForm', globals())

    def __init__(self, id):
        self.id = id
        self._user = None
        self._password = None
        self.host = None
        self.protocol = None


    security.declareProtected(CMFCorePermissions.ManagePortal, 'edit')
    def edit(self,
             user,
             password,
             password_confirm,
             host,
             protocol,
             remote_directory,
             RESPONSE=None):
        
        self._user = user
        
        if password_confirm == password:
            self._password = password
        else:
            raise CredentialError(" passwords do not match ")

        self.host = host.strip()

        if not protocol in getProtocolNames():
            raise ProtocolError(" Invalid Protocol %s"%protocol)

        self.protocol = protocol
        self.remote_directory = remote_directory.strip()


        if RESPONSE is not None:
            RESPONSE.redirect('deployment_settings')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'getUser')
    def getUser(self):
        return self._user

    # XXX check if fs dtml can access
    security.declarePrivate('getPassword')
    def getPassword(self):
        return self._password

    security.declareProtected(CMFCorePermissions.ManagePortal, 'getHost')
    def getHost(self):
        return self.host

    security.declareProtected(CMFCorePermissions.ManagePortal, 'getProtocol')
    def getProtocol(self):
        return getProtocol(self.protocol)


class ContentDeployment(Folder):

    meta_type = 'Content Deployment'

    manage_options = (

        {'label':'Overview',
        'action':'overview'},

        {'label':'Targets',
         'action':'manage_main'}

        )

    overview = DTMLFile('ui/ContentDeploymentOverview', globals())
    addDeploymentTargetForm = DTMLFile('ui/DeploymentTargetForm', globals())
    
    all_meta_types = (
        {'name':DeploymentTarget.meta_type,
         'action':addDeploymentTargetForm },
        )

    def __init__(self, id):
        self.id = id

    def addDeploymentTarget(self,
                            id,                            
                            user,
                            password,
                            password_confirm,
                            host,
                            remote_directory,
                            protocol,
                            RESPONSE=None):
        
        ob = DeploymentTarget(id)
        self._setObject(id, ob)
        ob = self._getOb(id)
        ob.edit(user, password, password_confirm, host, remote_directory, protocol)

        if RESPONSE is not None:
            RESPONSE.redirect("%s/manage_workspace"%id)

    def getProtocolTypes(self):
        return getProtocolNames()
        
    def deploy(self):

        # simple deployment
        structure = self.getContentOrganization().getActiveStructure()

        for target in self.objectValues('Deployment Target'):
            protocol = target.getProtocol()
            protocol.execute(target, structure)
        

    
#################################
# simple global protocol registry    

_protocols = {}
def registerProtocol(name, protocol):
    global _protocols
    _protocols[name]=protocol

def getProtocolNames():
    global _protocols
    return _protocols.keys()

def getProtocol(name):
    global _protocols
    return _protocols[name]()



class RsyncSshProtocol:

    def execute(self, target, structure):
        import pypect
        
        host = target.getHost()
        user = target.getUser()
        pass_ = target.getPassword()
        local_directory = structure.getMountPoint()
        remote_directory = target.getDirectory()


        #################################
        command = "rsync"

        short_noarg_options = [
            "a", # archive mode
            "z", # compress
            "t", # preserve mod times
            #"r", # recurse
            #"p", # preserve permissions
            #"u", # update, don't overwrite new files, i don't need this
            ]

        short_arg_options = [
            ("e", "ssh")  # ssh 
            ]

        # mainly aggressive deletion options
        # for retraction.
        long_options = [
            "--delete", # delete files which don't exist on the sender
            #"--ignore-errors"  # delete even if errors in transfer
            #"--force",  # force deletion of non empty dirs
            ]

        short = "-%s"%''.join(short_noarg_options)
        short_arg = "%s"%' '.join( map( lambda k,v: "-%s %s"%(k,v), short_arg_options))
        long_options = "%s"%' '.join(long_options)
        all_options = ' '.join( (short, short_arg, long_options) )
        #################################
        

        renderd_command = "%s %s %s %s"%(
            command,
            all_options,
            local_directory,
            "%s@%s:%s"%( user, host, remote_directory)
            )
        print renderd_command
        return
    
        conn = pypect.spawn(renderd_command)
        conn.expect('password:')
        conn.sendline(pass_)
        conn.expect_eof(30) # 30 s timeout
        
        if conn.isAlive():
            conn.kill(1)
        

                     
registerProtocol("RsyncSSH", RsyncSshProtocol)
