
from Namespace import *
from Products.CMFCore.Expression import Expression
from Products.CMFTopic import Topic
from Products.PageTemplates.Expressions import SecureModuleImporter, getEngine

class IContentIdentification(Interface):

    def getContent():
        """
        return the content that has been
        identified for deployment
        """

class IContentSource(Interface):

    def getContent():
        """
        returns a lazy sequence of content
        to be considered for deployment.
        realistically, these should be brains,
        exhibiting catalog metadata and a getObject
        method ;-) this is a bad interface ...
        """

class IContentFilter(Interface):

    def filter(content):
        """
        return boolean, true and the content
        continues through the filter chain
        """

class ContentIdentification(Folder):

    meta_type = 'Content Identification'

    manage_options = (

        {'label':'Overview',
         'action':'overview'},
        
        {'label':'Source',
         'action':'source/source'},

        {'label':'Filter Expressions',
         'action':'filters/manage_main'},

        {'label':'Filter Scripts',
         'action':'scripts/manage_main'},        

        {'label':'Policy',
         'action':'../overview'}
        )
    
    overview = DTMLFile('ui/IdentificationOverview', globals())

    allowed_meta_types = ()
    
    def __init__(self, id):
        self.id = id
        
    def getContent(self):
        # memory will explode :-(
        r = []
        w = r.append
        
        portal  = getToolByName(self, 'portal_url').getPortalObject()
        content = self.source.getContent()
        filters = self.filters.objectValues()
        scripts = self.scripts.objectValues()

        skip = 0
        for c in content:
            
            fc = getFilterExprContext(c,portal)

            for f in filters:
                if not f.filter(fc):
                    skip = 1
                    break
            if skip:
                skip = 0
                continue

            for s in scripts:
                if not s(fc):
                    skip = 1
                    break

            if skip:
                skip = 0
                continue

            w(c.getObject())
            
        return r


    def manage_afterAdd(self, item, container):
        self._setObject('source',  ContentSource('source'))
        self._setObject('filters', ContentExpressionFilterContainer('filters'))
        self._setObject('scripts', ContentScriptContainer('scripts'))
        
## XXX Todo make this a Topic
class ContentSource(SimpleItem):

    meta_type = 'Content Source'

    __implements__ = IContentSource

    manage_options = (

        {'label':'Source',
         'action':'source'},
        
        {'label':'Content Identification',
         'action':'../overview'},
        
        {'label':'Policy',
         'action':'../../overview'},                 
        )

    source = DTMLFile('ui/ContentSourceView', globals())

    def __init__(self, id):
        self.id = id

    def getContent(self):
        catalog = getToolByName(self, 'portal_catalog')
        objects = catalog()
        return objects


class ContentFilter(SimpleItem):

    meta_type = 'Content Filter'

    __implements__ = IContentFilter

    def __init__(self, id, text):
        self.id = id
        self.expression = Expression(text)
        
    def filter(self, context):
        return not not self.expression(context)

    def editFilter(self, expression_text, REQUEST=None):
        # perhaps better to edit... and remove volatiles
        self.expression = Expression(expression_text)
        

## XXX Todo make this ordered
class ContentExpressionFilterContainer(Folder):

    meta_type = 'Expression Filter Container'
    
    manage_options = (
        {'label':'Filter Expressions',
         'action':'manage_main'},
        
        {'label':'Content Identification',
         'action':'../overview'},
        
        {'label':'Policy',
         'action':'../../overview'},        
        )
        
    all_meta_types = (
        {'name':ContentFilter.meta_type,
        'class':ContentFilter,
        'permission':CMFCorePermissions.ManagePortal,
        'action':'addFilterForm'},
        )
    
    addFilterForm = DTMLFile('ui/ExpressionFilterAddForm', globals())
    
    def __init__(self, id):
        
        self.id = id

    def addFilter(self, id, text, REQUEST=None):
        """ """
        self._setObject(id, ContentFilter(id, text))
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect('manage_main')


## XXX Todo make this ordered
class ContentScriptContainer(Folder):

    meta_type = 'Script Container'

    manage_options = (
        {'label':'Filter Scripts',
         'action':'manage_main'},
        
        {'label':'Content Identification',
         'action':'../overview'},
        
        {'label':'Policy',
         'action':'../../overview'},        
        )
    def all_meta_types(self):
        """ adapation of zopelab recipe """
        
        import Products
        return [m for m in Products.meta_types if m['name']=='Python Script']
    


def getFilterExprContext(object_memento, portal):

    data = {
        'portal_url':   portal.absolute_url(),
        'memento':      object_memento,
        'portal':       portal,
        'nothing':      None,
        'request':      getattr( portal, 'REQUEST', None ),
        'modules':      SecureModuleImporter,
        }
        
    return getEngine().getContext(data)


    
