from Namespace import *
from Products.CMFCore.Expression import Expression
from Products.PageTemplates.Expressions import SecureModuleImporter, getEngine
from utils import file2string

class InvalidSkinName(Exception): pass
    
class ContentMastering(Folder):

    meta_type = 'Content Mastering' 

    manage_options = (

        {'label':'Overview',
         'action':'overview'},
        
        {'label':'Skin',
         'action':'skin'},

        {'label':'SiteRoot',
         'action':'root'},        

        {'label':'MimeMapping',
         'action':'mime/manage_main'},        

        {'label':'Policy',
         'action':'../overview'}

        )

    overview = DTMLFile('ui/ContentMasteringOverview', globals())
    skin     = DTMLFile('ui/ContentMasteringSkin', globals())
    root     = DTMLFile('ui/ContentMasteringRoot', globals())

    def __init__(self, id):
        
        self.id = id
        self.skn_name = None

    def editRoot(self, enable, server, server_path, port, protocol, RESPONSE=None):
        """ """
        
        self.site_root.edit(enable, server, server_path, protocol, port)

        if RESPONSE is not None:
            RESPONSE.redirect('overview')

    def editSkin(self, enable, skin_name, RESPONSE=None):
        """ """
        
        skin_tool = getToolByName(self, 'portal_skins')
        skin = skin_tool.getSkinByName(skin_name.strip())
        
        if skin is None:
            raise InvalidSkinName("%s is not a valid skin"%str(skin_name))

        self.site_skin.edit(enable, skin_name)

        if RESPONSE is not None:
            RESPONSE.redirect('overview')

    def manage_afterAdd(self, item, container):

        ob = MimeMappingContainer('mime')
        self._setObject('mime', ob)

        ob = SiteChainRoot()
        self._setObject('site_root', ob)

        ob = SiteChainSkin()
        self._setObject('site_skin', ob)

    #################################
    def precook(self, descriptor):

        portal = getToolByName(self, 'portal_url').getPortalObject()        
        c = descriptor.getContent()
        ec = getMimeMappingExprContext(c, portal)
        fc = getMimeFilterExprContext(c, portal)

        mappings = self.mime.objectValues()

        valid = 0
        for m in mappings:
            if m.valid(fc):
                valid = 1
                break

        if valid:
            descriptor.setExtension(m.extension(ec))
            descriptor.setRenderMethod(m.view_method)
            return 1
        
        return None

    def cook(self, descriptor):

        c = descriptor.getContent()
        vm = descriptor.getRenderMethod()
        render = getattr(c, vm, None) or c.__call__

        #print 'rendering', c.getId(), vm, render
        
        if getattr(aq_base(render), 'isDocTemp', 0):
            descriptor.setRendered(apply(render, (self, self.REQUEST)))
        elif hasattr(c.aq_base, 'precondition'): # only reliable way to distinguish file like objs...
            descriptor.setRendered(file2string(c))
        else:
            descriptor.setRendered(render())

    #################################
    def setup(self):
        self.site_root.lock()
        self.site_skin.lock()
        
    def tearDown(self):
        self.site_root.unlock()
        self.site_skin.unlock()

class MimeExtensionMapping(SimpleItem):

    meta_type = 'Mime Extension Mapping'

    view_method = ''

    manage_options = (
        {'label':'Mapping',
         'action':'editMappingForm'},
        {'label':'All Mappings',
         'action':'../manage_main'},
        {'label':'Mastering',
         'action':'../../overview'}
        )
    
    editMappingForm = DTMLFile('ui/MimeExtensionMappingEditForm', globals())
    
    def __init__(self, id, extension_expression, condition, view_method):
        self.id = id
        
        self.extension = Expression(extension_expression)
        self.extension_text = extension_expression
        self.condition = Expression(condition)
        self.condition_text = condition
        self.view_method = view_method.strip()
        
    def valid(self, context):
        return not not self.condition(context)

    def extension(self, context):
        return self.extension(context)

    def editMapping(self, extension_expression, condition, view_method, RESPONSE=None):
        """ """
        self.extension = Expression(extension_expression)
        self.extension_text = extension_expression
        self.condition = Expression(condition)
        self.condition_text = condition
        self.view_method = view_method.strip()

        if RESPONSE is not None:
            RESPONSE.redirect('../manage_main')

class MimeMappingContainer(Folder):

    meta_type = 'Mime Mapping Container'

    manage_options = (
        {'label':'Mappings',
         'action':'manage_main'},

        {'label':'Mastering',
         'action':'../overview'},        

        {'label':'Policy',
         'action':'../../overview'}
        )

    addMappingForm = DTMLFile('ui/MimeExtensionMappingAddForm', globals())

    all_meta_types = (
        {'name':MimeExtensionMapping.meta_type,
         'action':'addMappingForm'},
        )

    def __init__(self, id):
        self.id = id

    def addMimeMapping(self, id, extension_expression, condition, view_method, RESPONSE=None):
        """ """
        mapping = MimeExtensionMapping(id,
                                       extension_expression,
                                       condition,
                                       view_method)

        self._setObject(id, mapping)

        if RESPONSE is not None:
            RESPONSE.redirect('manage_main')
        

def getMimeFilterExprContext(object, portal):
    
    data = {
        'object':       object,
        'portal':       portal,
        'nothing':      None,
        'request':      getattr( object, 'REQUEST', None ),
        'modules':      SecureModuleImporter,
        }

    return getEngine().getContext(data)    
    
    
def getMimeMappingExprContext(object, portal):

    data = {
        'object':       object,
        'portal':       portal,
        'nothing':      None,
        'request':      getattr( object, 'REQUEST', None ),
        'modules':      SecureModuleImporter,
        }
        
    return getEngine().getContext(data)

#################################
# both the chain classes are hacks

class SiteChainSkin(SimpleItem):
    """

    changes the skin mid request,

    """

    meta_type = 'Site Chain Skin'
    id = 'site_skin'
    
    _v_active = None
    _v_saved_skin_data = None

    
    def __init__(self):
        self.enable = None
        self.skin_name = None

    def edit(self, enable, skin_name):
        
        self.enable = not not int(enable)
        self.skin_name = skin_name

    def lock(self):

        if not self.enable or self._v_active or not self.skin_name:
            print 'aborting chain skin'
            return

        portal = getToolByName(self, 'portal_url').getPortalObject()
        req = getattr(self, 'REQUEST', {})
        self._v_saved_skin_data = portal._v_skindata
        portal._v_skindata = (req,
                              self.getSkinByName(self.skin_name),
                              {})
        self._v_active = 1
        
    def unlock(self):

        if not self._v_active:
            return

        portal = getToolByUrl(self, 'portal_url').getPortalObject()
        portal._v_skindata = self._v_saved_skin_data

        self._v_saved_skin_data = None
        self._v_active = None
        

class SiteChainRoot(SimpleItem):
    """
    it creates a site root temporarily

    for now i'm not bothering to define an unlock,
    because this is way too hackish to safely be
    certain of unlocking, and it has no byproducts
    beyond the current request.
    
    """
    meta_type = 'Site Chain Root'
    id = 'site_root'

    _v_active = None
    
    def __init__(self):
        
        self.enable = None # flag to use or not
        self.protocol = None
        self.port = None
        self.server = None        
        self.server_path = None
        
    def edit(self, enable, server, server_path, protocol, port):
        
        self.enable = not not int(enable)
        self.server = server.strip()
        self.server_path = server_path.strip()
        self.protocol = protocol.strip()
        self.port = port.strip()

    def lock(self):

        if not self.enable or not self.server or not self.server_path or self._v_active:
            print 'aborting chain root'
            return

        # XXX This is the voodoo
        req = self.REQUEST
        ob = self.getContentOrganization().getActiveStructure().getCMFMountPoint()
        
        other = req.other
        path  = filter(None, self.server_path.split('/'))


        req.setServerURL(protocol=self.protocol, hostname=self.server, port=self.port)
        req._script[:] = path
        del req._steps[:]
        other['VirtualRootPhysicalPath'] = ob.getPhysicalPath()
                       
        req._resetURLS()

        self._v_active = 1
        
        return

    def unlock(self):

        if not self.active:
            return

        # XXX
        
        self._v_active = None
        
        return
        
            
            
