from os import sep
from Namespace import *
from FileStructure import RootDirectory, descendant_folder_collector

class InvalidCMFMountPoint(Exception): pass

class ContentOrganization(Folder):

    meta_type = 'Content Organization'

    security = ClassSecurityInfo()

    default_manage_options = (
        
        {'label':'Overview',
         'action':'overview'},
        
        {'label':'Structure',
         'action':'cmfstructure/filesystem_mount'},

        {'label':'Organize',
         'action':'view_organize'},

        {'label':'Policy',
         'action':'../overview'}

        )

    advanced_manage_options = (

        {'label':'Overview',
         'action':'overview'},
        
        {'label':'Structure',
         'action':'structure/manage_main'},

        {'label':'Organize',
         'action':'organize/manage_main'},

        {'label':'Policy',
         'action':'../overview'}

        )

    organization_policy = 'default'

    def manage_options(self):
        if self.organization_policy == 'default':
            return self.default_manage_options
        else:
            return self.advanced_manage_optiosn

    manage_options = ComputedAttribute(manage_options)

    view_structure = DTMLFile('ui/ContentOrganizationStructure', globals())
    view_organize  = DTMLFile('ui/ContentOrganizationOrganize',  globals())
    overview  = DTMLFile('ui/ContentOrganizationOverview',  globals())

    def __init__(self, id):
        self.id = id

    security.declareProtected(CMFCorePermissions.ManagePortal, 'changeMode')
    def changeMode(self, RESPONSE=None):
        ' '
        if self.organization_policy == 'default':
            self.organization_policy = 'advanced'
        else:
            self.organization_policy = 'default'
            
        if RESPONSE is not None:
            RESPONSE.redirect('overview')

    security.declarePrivate('getActiveStructure')
    def getActiveStructure(self):

        if self.organization_policy == 'default':
            return self.cmfstructure
        else:
            return self.structure

    security.declarePrivate('mount')
    def mount(self):
        structure = self.getActiveStructure()
        structure.mount()

    security.declarePrivate('unmount')
    def unmount(self):
        structure = self.getActiveStructure()
        structure.unmount()

    security.declarePrivate('store')
    def store(self, descriptor):
        filename = descriptor.getFileName()
        structure = self.getActiveStructure()
        
        location = sep.join(  ( structure.getContentPath(descriptor.getContent()),
                                filename)
                              )

        print 'store', location, descriptor.getContent().Format()
    
        content = descriptor.getContent()
        rendered = descriptor.getRendered()
        
        if content.Format().split('/')[0]=='text':
            fh = open(location, 'w')
        else:
            fh = open(location, 'wb')
            
        fh.write(rendered)
        fh.close()

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        self._setObject('structure', AdvancedContentStructure('structure'))
        self._setObject('cmfstructure',ContentStructure('cmfstructure'))
        self._setObject('organize', StructureMapping('organize'))

InitializeClass(ContentOrganization)

class ContentStructure(RootDirectory):

    meta_type = 'ContentStructure'

    manage_options = (
        
        {'label':'FS Mount',
         'action':'filesystem_mount'},

        {'label':'CMF Mount',
         'action':'cmf_mount'},

        {'label':'Organization',
         'action':'../overview'},
        
        {'label':'Policy', 
         'action':'../../overview'}  

        )

    cmf_mount = DTMLFile('ui/CMFMountForm', globals())
    cmf_mount_point = '/'

    def mount(self):

        root  = self.getCMFMountPoint()
        rlen = len(root.getPhysicalPath())

        for f in descendant_folder_collector(root):
            fp = f.getPhysicalPath()[rlen:]
            local = self
            for fps in fp:
                next = getattr(local, fps, None)
                if next is None:
                    local.addDirectory(fps)
                local = next

        ContentStructure.inheritedAttribute('mount')(self)

    def unmount(self):
        # clear 
        self.manage_delObjects(self.objectIds())

    def setCMFMountPoint(self, cmf_mount_point, RESPONSE=None):
        """ """

        # get rid of leading and trailing slashes
        cmf_mount = '/'.join(filter(None,cmf_mount_point.split('/')))
        self.cmf_mount_point = cmf_mount

        # make sure its valid
        try:
            self.getCMFMountPoint()
        except:
            raise InvalidCMFMountPoint("invalid mount point %s"%cmf_mount)

        if RESPONSE is not None:
            RESPONSE.redirect('cmf_mount')
                    
    def getCMFMountPoint(self):
        
        root = getToolByName(self, 'portal_url').getPortalObject()
        path = filter(None, self.cmf_mount_point.split('/'))
        for p in path:
            root = getattr(root, p)        
        return root
        
    def getContentPath(self, content):
        # means we want the full fs path

        rlen = len(self.getCMFMountPoint().getPhysicalPath())
        return sep.join ( (self.mount_point,
                           sep.join(content.getPhysicalPath()[rlen:][:-1]) )
                          )
                          


#################################
# punt on this cause i don't need, the design is
# also radically different. the content is sieved
# onto this structure before rendering.

class AdvancedContentStructure(RootDirectory):

    meta_type = 'Content Structure'

    manage_options = (
        
        {'label':'Structure',
         'action':'manage_main'},

        {'label':'Mount',
         'action':'manage_mount'},

        {'label':'Content Organization',
         'action':'../overview'},
        
        {'label':'Policy', 
         'action':'../../overview'}  

        )


class StructureMapping(Folder):
    
    meta_type = 'Structure Mapping'

    def __init__(self, id):
        self.id = id

    def getContentPath(self, content):
        raise NotImplemented (" not ready yet ")

class MappingRule(SimpleItem):
    def __init__(self, id, target, expression):
        pass
