import DefaultConfiguration
from Namespace import *
from Descriptor import ContentDescriptor

class IDeploymentPolicy(Interface):


    def getContentIdentification():
        """
        return the component that
        handles identification of
        content that will be deployed
        """
        
    def getContentOrganization():
        """
        return the component that handles
        organizing content for deployment.
        """

    def getContentMastering():
        """
        return the component that handles
        cooking the content for the target
        deployment
        """

    def getContentDeployment():
        """
        return the component that handles
        deploying content to the target
        systems
        """

    def getDeploymentPolicy():
        """
        return the deployment policy
        """

    def execute():
        """
        execute the policy, see documentation
        for more information on the various
        stages of execution
        """

    def setActive(active_flag):
        """
        makes the policy and active and therefore
        executable
        """

    def isActive():
        """
        deactivate this policy
        """

class DeploymentPolicy(Folder):

    meta_type = 'Deployment Policy'

    __implements__ = IDeploymentPolicy

    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.ManagePortal)

    manage_options = (

        {'label':'overview',
         'action':'overview'},
        
        {'label':'Identification',
         'action':'%s/manage_workspace'%DefaultConfiguration.ContentIdentification},

        {'label':'Organization',
         'action':'%s/manage_workspace'%DefaultConfiguration.ContentOrganization},

        {'label':'Templating',
         'action':'%s/manage_workspace'%DefaultConfiguration.ContentMastering},

        {'label':'Deployment', 
         'action':'%s/manage_workspace'%DefaultConfiguration.ContentDeployment},

        {'label':'Tool', 
         'action':'../manage_main'}
        
        ) 

    overview = DTMLFile('ui/PolicyOverview', globals())
    identification = DTMLFile('ui/ContentIdentification', globals())
    _active = 1
    
    icon = 'misc_/CMFDeployment/policy.png'
    
    def __init__(self, id):
        self.id = id
        self.active = 0

    def getContentIdentification(self):
        return self._getOb(DefaultConfiguration.ContentIdentification)

    def getContentOrganization(self):
        return self._getOb(DefaultConfiguration.ContentOrganization)

    def getContentMastering(self):
        return self._getOb(DefaultConfiguration.ContentMastering)

    def getContentDeployment(self):
        return self._getOb(DefaultConfiguration.ContentDeployment)

    def getDeploymentPolicy(self):
        return self

    def setActive(self, flag, RESPONSE=None):
        self._active = not not flag
        if RESPONSE is not None:
            RESPONSE.redirect('overview')

    def isActive(self):
        return self._active

    def execute(self, RESPONSE):
        """ """

        if not self.isActive():
            return 2
        
        #################################
        # XXX this has much that is ugly...
        
        source    = self.getContentIdentification()
        structure = self.getContentOrganization()
        mastering = self.getContentMastering()
        deployer  = self.getContentDeployment()

        # import pdb; pdb.set_trace()
        structure.mount()

        # this is ugly
        store = structure.cmfstructure
        cmpt  = structure.cmfstructure.cmf_mount_point
        
        mastering.setup()

        # we do string based uri replacement
        url_lookup = {}
        descriptors = []
        w = descriptors.append
         
        try:
            content = source.getContent()
            #filter(None, map(mastering.precook, map(ContentDescriptor, content)))
            
            for ci in content:
                if cmpt not in ci.getPhysicalPath():
                    continue

                d = ContentDescriptor(ci)
                if not mastering.precook(d):
                    c = d.getContent()
                    print 'skipped', c.portal_type, c.absolute_url()
                    #print 'skipped', d.getContent().getPhysicalPath()
                    #print 'skip url', d.getContent().absolute_url().strip()
                    continue

                #import pdb; pdb.set_trace()
                url = ci.absolute_url()
                base_url = url[:url.rfind('/')]
                url_lookup[url]='/'.join( (base_url, d.getFileName() ) )
                w(d)



            rset = url_lookup.items()
            rset.sort()
            rset.reverse()
            for k,v in rset:
                print 'lookup translation', k, v                
                
            for d in descriptors:
                # we really should raise an error 
                mastering.cook(d)
                rendered = d.getRendered()
                for k,v in url_lookup.items():
                    rendered = rendered.replace('href="%s"'%k, 'href="%s"'%v)
                d.setRendered(rendered)
                structure.store(d)
                del d
                
            del url_lookup
            del descriptors

                
        finally:
            mastering.tearDown()
            
        deployer.deploy()
        
        #################################

        return "worked"


    def manage_afterAdd(self, item, container):
        import Install
        Install.install(self)
    
InitializeClass(DeploymentPolicy)
    

